/*--

   Copyright (c) 2004 Ambros Marzetta and David Bremner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

--*/
/* Job interface example
   Ambros Marzetta

 * $Source$
 * $Revision: 5167 $
 * $Date: 2004-10-20 13:20:33 -0300 (Wed, 20 Oct 2004) $
 * $Log$
 * Revision 1.3  2004/10/20 16:20:31  bremner
 * added copyright notice and GPL boilerplate
 *
 * Revision 1.2  2000/11/15 16:14:51  bremner
 *
 * committing patched version of zram source
 *
 */

#include <stdio.h>
#include "z_base.h"
#include "z_par.h"

#ifndef SEQUENTIAL

#define TEST1 22
#define TEST2 6327388

static int example_NodeTag;

typedef struct example_Node {
  Z_NODE_HEADER;
  char cc[1000];
  int8 x;
} *example_NodeP;

static int example_jobnum, example_collect_jobnum;

static void example_job(void)
{
  example_NodeP n, m;
  int32 tel;
  
  tel = z_GetMINTBuffer();
  assert(tel == TEST2);
  n = (example_NodeP) z_GetNodeBuffer();
  m = (example_NodeP) z_GetNodeBuffer();
  printf("[%2d]example_job: tel = %ld, x = %ld, m = %lx\n",
	 z_myproc, (long)tel, (long)n->x, (long)m);

  if (z_myproc == 0) {		/* send result to host */
    n->x = 45;
    z_AddTagBuffer(39);
    z_AddNodeBuffer((z_NodeP) n);
    z_AddMINTBuffer(95);
  }
} /* example_job */

static void example_collect_job(void)
{
    example_NodeP n;
    
  printf("[%2d]Collect job started\n", z_myproc);
  n = (example_NodeP)
         z_NewNode(example_NodeTag, sizeof(struct example_Node));
  n->x = TEST1;
    z_AddNodeBuffer((z_NodeP) n);
  z_AddTagBuffer(100 + z_myproc);
  z_AddMINTBuffer(1000 + z_myproc);
    n->x ++;
    z_FreeNode((z_NodeP) n);
} /* example_collect_job */

void initproc(void)
{
  /* This function is called once on every processing element. */

  example_jobnum = z_InstallJob(&example_job);
  example_collect_jobnum = z_InstallJob(&example_collect_job);
  example_NodeTag = z_InstallClass(example_Node, 0, 0, 0);
} /* initproc */

#ifndef PEonly

static void Result(int pe)
{
    example_NodeP n;

    n = (example_NodeP)z_GetNodeBuffer();
  printf("Result from PE %d: %3ld %4ld\n",
	 pe, (long)z_GetTagBuffer(), (long)z_GetMINTBuffer());
    assert(n->x == TEST1);
    z_FreeNode((z_NodeP) n);
} /* Result */

int mmain(int argc, char **argv, char **envp)
{
  /* main program */
  example_NodeP root, sol;

  /* example for normal job (result on PE 0) */
  root = (example_NodeP)
         z_NewNode(example_NodeTag, sizeof(struct example_Node));
  root->x = 68;
  z_AddMINTBuffer(TEST2);
  z_AddNodeBuffer((z_NodeP)root);
  z_AddNodeBuffer(NULL);
  z_StartParallelJob(example_jobnum, NULL);
  z_WaitWork();
  sol = (example_NodeP) z_GetNodeBuffer();
  printf("Result: tag = %ld, x = %ld, 95 = %ld\n\n",
	 (long)z_GetTagBuffer(), (long)sol->x, (long)z_GetMINTBuffer());

  /* example for collect job (result on all PEs) */
  z_StartParallelJob(example_collect_jobnum, Result);
  z_WaitWork();
  
  return 0;
} /* mmain */

#endif /* PEonly */

#else /* SEQUENTIAL */

void initproc(void)
{
} /* initproc */

int mmain(int argc, char **argv, char**envp)
{
  printf("job_example must not be compiled with -DSEQUENTIAL.\n");
  return 0;
} /* mmain */

#endif /* SEQUENTIAL */
