/*--

   Copyright (c) 2004 Ambros Marzetta and David Bremner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

--*/
/* vc_br.c (Vertex cover branching rules)
   Ambros Marzetta
   94-10-17
   For more information see our WWW page at
   
         http://wwwjn.inf.ethz.ch/ambros/zram.html
	 
 * $Source$
 * $Revision: 5167 $
 * $Date: 2004-10-20 13:20:33 -0300 (Wed, 20 Oct 2004) $
 * $Log$
 * Revision 1.3  2004/10/20 16:20:32  bremner
 * added copyright notice and GPL boilerplate
 *
 * Revision 1.2  2000/11/15 16:14:52  bremner
 *
 * committing patched version of zram source
 *
 * Revision 1.1  1998/04/15  08:47:29  marzetta
 * Initial revision
 *
 *
 */

#include <stddef.h> /* NULL */
#include "vc_br.h"
#include "vc_lb.h"

static MINT *degree1list;
static long degree1listsize;

static void MarkAllNeighbours(vc_GlobalP global, vc_NodeP np,
			      MINT pt, MINT **head)
{
  /* Mark all neighbours of pt. */
  MINT other, *adj;
  
  assert(np->tag == vc_NodeTag);
  adj = vc_adjPt(global, pt);
  while (*adj != ADJ_LAST) {
    other = vc_pt(*adj);
    if vc_active(np, other)
      vc_Mark(global, np, other, head);
    adj ++;
  } /* while */
} /* MarkAllNeighbours */

static void vc_Simplify(vc_GlobalP global, vc_NodeP np,
			MINT **head, MINT **tail)
{
  /* Remove points of degree 1 */
  
    MarkHeadTail(global, np, head, tail);
} /* vc_Simplify */

void vc_Branch(vc_GlobalP global, vc_NodeP np, vc_NodeP upperBound,
	       z_bb_DoProcP whatToDo, void *ref)
{
  /* Allocate nodes for children of np. */
  /* Call whatToDo(child, ref) for all children of np. */
  /* whatToDo is responsible for deallocating the children. */
  /* whatToDo may call vc_Branch recursively. */
  /* np must not be a solution. */
  
  vc_NodeP np_marked, np_unmarked;
  MINT maxdegreepoint, *head, *tail;

  assert(global);
  maxdegreepoint = vc_MaxDegreePoint(np);

  z_ReSizePointer((void *)&degree1list,
		&degree1listsize, np->vc_numPt * sizeof(MINT));
  head = tail = degree1list;
  np_marked = (vc_NodeP) z_NewCopyNode((z_NodeP) np);
  vc_Mark(global, np_marked, maxdegreepoint, &head);
  vc_Simplify(global, np_marked, &head, &tail);
  if (vc_IsSolution(np_marked)) UnmarkPoints(global, np_marked);
  np_marked->vc_lb = vc_LowerBound(global, np_marked);
  whatToDo((z_NodeP) np_marked, ref);

  z_ReSizePointer((void *)&degree1list,
		&degree1listsize, np->vc_numPt * sizeof(MINT));
				/* might be changed by whatToDo */
  head = tail = degree1list;
  np_unmarked = (vc_NodeP) z_NewCopyNode((z_NodeP) np);
  MarkAllNeighbours(global, np_unmarked, maxdegreepoint, &head);
  vc_Simplify(global, np_unmarked, &head, &tail);
  if (vc_IsSolution(np_unmarked)) UnmarkPoints(global, np_unmarked);
  np_unmarked->vc_lb = vc_LowerBound(global, np_unmarked);
  whatToDo((z_NodeP) np_unmarked, ref);
} /* vc_Branch */
